-- Copyright 2024 DJ WoodZ

-- Permission is hereby granted, free of charge, to any person obtaining a copy
-- of this software and associated documentation files (the "Software"), to deal
-- in the Software without restriction, including without limitation the rights
-- to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is
-- furnished to do so, subject to the following conditions:

-- The above copyright notice and this permission notice shall be included in all
-- copies or substantial portions of the Software.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-- IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-- FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-- AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-- LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-- OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-- SOFTWARE.

---Event for Fill Type setting update
EnhancedEconomySettingsUpdateEvent = {}
local EnhancedEconomySettingsEvent_mt = Class(EnhancedEconomySettingsUpdateEvent, Event)
InitEventClass(EnhancedEconomySettingsUpdateEvent, 'EnhancedEconomySettingsUpdateEvent');

---Create instance of Event class
-- @return table self instance of class event
function EnhancedEconomySettingsUpdateEvent.emptyNew()
  local self = Event.new(EnhancedEconomySettingsEvent_mt)
  return self
end

---Create new instance of event
-- @param table enhancedEconomySettings the economy settings
function EnhancedEconomySettingsUpdateEvent.new(enhancedEconomySettings)
  local self = EnhancedEconomySettingsUpdateEvent.emptyNew()
  self.enhancedEconomySettings = enhancedEconomySettings
  return self
end

--- Called on sending side
-- @param integer streamId streamId
-- @param integer connection connection
function EnhancedEconomySettingsUpdateEvent:writeStream(streamId, connection)
  for fillType, value in pairs(self.enhancedEconomySettings) do
    streamWriteString(streamId, fillType)
    streamWriteFloat32(streamId, value)
  end
end

--- Called on receiving side
-- @param integer streamId streamId
-- @param integer connection connection
function EnhancedEconomySettingsUpdateEvent:readStream(streamId, connection)
  self.enhancedEconomySettings = {};

  while true do
    local fillType = streamReadString(streamId);
    local value = streamReadFloat32(streamId);

    -- if fillType is valid
    if (fillType ~= nil) then
      self.enhancedEconomySettings[fillType] = value;
    else
      break
    end
  end

  self:run(connection)
end

--- Run action
-- @param integer connection connection
function EnhancedEconomySettingsUpdateEvent:run(connection)
  -- whether on server or client, update economy
  EnhancedEconomySettings:updateEconomy(self.enhancedEconomySettings)

  -- if on server
  if g_server ~= nil then
    -- also broadcast to all clients
    g_server:broadcastEvent(EnhancedEconomySettingsUpdateEvent.new(self.enhancedEconomySettings), false)
  end
end

---To be called on client. Sends event to server to broadcast update to all clients.
-- @param table enhancedEconomySettings the economy settings
function EnhancedEconomySettingsUpdateEvent:sendEvent(enhancedEconomySettings)
  -- if on client
  if g_server == nil then
    g_client:getServerConnection():sendEvent(EnhancedEconomySettingsUpdateEvent.new(enhancedEconomySettings))
  end
end
